import { exportDataGrid as exportToExcel } from 'devextreme/excel_exporter'
import { saveAs } from 'file-saver'
import { jsPDF } from 'jspdf'
import autoTable from 'jspdf-autotable'
import { Workbook } from 'exceljs'
import { setHeaderStyle } from '@/report/utils/xlsx'
import { formatNumber, formatPercentage } from '@/utils/numbers'
import { getMonthName } from '@/utils/texts'
import { exportToWord } from './doc/MonalisaGR_ResponseTimeGangguan_DOC'

const reportName = 'Response Time (RPT) Gangguan'
const fontSize = 5

const groupingData = (data: any, reportMeta: any) => {
  const groupedData: any = {}

  const reportMetaMapping: any = {
    ulp: 'nama_ulp',
    up3: 'nama_ulp',
    uid: 'nama_up3',
    regional: 'nama_uid',
    default: 'nama_regional'
  }

  let key = 'nama_regional'
  for (const prop in reportMetaMapping) {
    if (reportMeta[prop] && reportMeta[prop].id != 0) {
      key = reportMetaMapping[prop]
      break
    }
  }

  data.forEach((item: any) => {
    const groupKey = item[key]

    if (!groupedData[groupKey]) {
      groupedData[groupKey] = { data: [] }
    }

    groupedData[groupKey].data.push(item)
  })

  for (const key in groupedData) {
    const data = groupedData[key].data

    data.forEach((item: any) => {
      if (!groupedData[key].summary) {
        groupedData[key].summary = {
          mom_bulan_kemarin: 0,
          mom_bulan_ini: 0,
          persen_mom: [],
          yoy_tahun_kemarin: 0,
          yoy_tahun_ini: 0,
          persen_yoy: []
        }
      }

      groupedData[key].summary.mom_bulan_kemarin += item.mom_bulan_kemarin
      groupedData[key].summary.mom_bulan_ini += item.mom_bulan_ini
      groupedData[key].summary.persen_mom.push(item.persen_mom)
      groupedData[key].summary.yoy_tahun_kemarin += item.yoy_tahun_kemarin
      groupedData[key].summary.yoy_tahun_ini += item.yoy_tahun_ini
      groupedData[key].summary.persen_yoy.push(item.persen_yoy)
    })
  }

  return groupedData
}

const getTitle = (reportMeta: any) => {
  if (reportMeta.ulp.id != 0) {
    return reportMeta.ulp.name
  }

  if (reportMeta.up3.id != 0) {
    return reportMeta.up3.name
  }

  if (reportMeta.uid.id != 0) {
    return reportMeta.uid.name
  }

  if (reportMeta.regional.id != 0) {
    return reportMeta.regional.name
  }

  return 'NASIONAL'
}

const formatData = (rawData: any, reportMeta: any) => {
  const formattedData: any = []

  const total: any = {
    mom_bulan_kemarin: [],
    mom_bulan_ini: [],
    yoy_tahun_kemarin: [],
    yoy_tahun_ini: []
  }

  const data = groupingData(rawData, reportMeta)
  const title = getTitle(reportMeta)

  formattedData.push([{ content: title, colSpan: 7, styles: { fontStyle: 'bold' } }])

  let no = 1
  for (const key in data) {
    const summary = data[key].summary

    formattedData.push([
      { content: no++, styles: { halign: 'right' } },
      key,
      formatNumber(summary.mom_bulan_kemarin),
      formatNumber(summary.mom_bulan_ini),
      formatPercentage(
        !summary.mom_bulan_ini || !summary.mom_bulan_kemarin
          ? '0%'
          : ((summary.mom_bulan_kemarin - summary.mom_bulan_ini) / summary.mom_bulan_kemarin) * 100
      ),
      formatNumber(summary.yoy_tahun_kemarin),
      formatNumber(summary.yoy_tahun_ini),
      formatPercentage(
        !summary.yoy_tahun_ini || !summary.yoy_tahun_kemarin
          ? '0%'
          : ((summary.yoy_tahun_kemarin - summary.yoy_tahun_ini) / summary.yoy_tahun_kemarin) * 100
      )
    ])

    total.mom_bulan_kemarin.push(summary.mom_bulan_kemarin)
    total.mom_bulan_ini.push(summary.mom_bulan_ini)
    total.yoy_tahun_kemarin.push(summary.yoy_tahun_kemarin)
    total.yoy_tahun_ini.push(summary.yoy_tahun_ini)
  }

  const totalMoMBulanKemarin =
    total.mom_bulan_kemarin.reduce((a: number, b: number) => a + b, 0) /
    total.mom_bulan_kemarin.length
  const totalMoMBulanIni =
    total.mom_bulan_ini.reduce((a: number, b: number) => a + b, 0) / total.mom_bulan_ini.length
  const totalYoYTahunKemarin =
    total.yoy_tahun_kemarin.reduce((a: number, b: number) => a + b, 0) /
    total.yoy_tahun_kemarin.length
  const totalYoYTahunIni =
    total.yoy_tahun_ini.reduce((a: number, b: number) => a + b, 0) / total.yoy_tahun_ini.length

  formattedData.push([
    { content: 'RATA-RATA', colSpan: 2, styles: { fontStyle: 'bold' } },
    formatNumber(totalMoMBulanKemarin),
    formatNumber(totalMoMBulanIni),
    formatPercentage(
      !totalMoMBulanIni || !totalMoMBulanKemarin
        ? '0%'
        : ((totalMoMBulanKemarin - totalMoMBulanIni) / totalMoMBulanKemarin) * 100
    ),
    formatNumber(totalYoYTahunKemarin),
    formatNumber(totalYoYTahunIni),
    formatPercentage(
      !totalYoYTahunIni || !totalYoYTahunKemarin
        ? '0%'
        : ((totalYoYTahunKemarin - totalYoYTahunIni) / totalYoYTahunKemarin) * 100
    )
  ])

  return formattedData
}

const exportToPDF = (reportMeta: any, rawData: any, preview: boolean = false) => {
  const day = new Date().toLocaleString('id-ID', { weekday: 'long' })
  const date = new Date().getDate()
  const month = new Date().toLocaleString('id-ID', { month: 'long' })
  const year = new Date().getFullYear()
  const data = formatData(rawData, reportMeta)
  const doc = new jsPDF({
    orientation: 'landscape'
  })

  const img = new Image()

  img.src = '/assets/images/pln-icon.png'
  doc.addImage(img, 'png', 13.5, 9, 9.5, 13)

  autoTable(doc, {
    head: [
      ['PT. PLN(Persero)', '', ''],
      [
        'REGIONAL',
        ':',
        reportMeta.regional
          ? reportMeta.regional.name.toUpperCase()
          : 'Semua Regional'.toUpperCase()
      ],
      [
        { content: 'UNIT INDUK', styles: { cellWidth: 40 } },
        { content: ':', styles: { cellWidth: 1 } },
        reportMeta.uid
          ? reportMeta.uid.name.toUpperCase()
          : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
      ],
      [
        'UNIT PELAKSANA PELAYANAN PELANGGAN',
        ':',
        reportMeta.up3
          ? reportMeta.up3.name.toUpperCase()
          : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
      ],
      [
        'UNIT LAYANAN PELANGGAN',
        ':',
        reportMeta.ulp
          ? reportMeta.ulp.name.toUpperCase()
          : 'Semua Unit Layanan Pelanggan'.toUpperCase()
      ]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold'
    },
    theme: 'plain',
    startY: 10,
    margin: {
      left: 25
    }
  })

  autoTable(doc, {
    head: [[`${reportName}`.toUpperCase()], [reportMeta.periode]],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    startY: 23
  })

  autoTable(doc, {
    head: [
      [
        {
          content: 'No',
          rowSpan: 3
        },
        {
          content: 'Nama Unit',
          rowSpan: 3
        },
        {
          content: 'Jumlah RCT Kali Gangguan',
          colSpan: 6
        }
      ],
      [
        {
          content: 'MOM',
          colSpan: 3
        },
        {
          content: 'YOY',
          colSpan: 3
        }
      ],
      [
        `${getMonthName(reportMeta.lastMonth)} ${reportMeta.lastYearMoM}`,
        `${getMonthName(reportMeta.currentMonth)} ${reportMeta.currentYear}`,
        '%',
        `s.d ${getMonthName(reportMeta.currentMonth)} ${reportMeta.lastYear}`,
        `s.d ${getMonthName(reportMeta.currentMonth)} ${reportMeta.currentYear}`,
        '%'
      ]
    ],
    body: data,
    styles: {
      fontSize,
      cellPadding: 1,
      lineColor: [0, 0, 0],
      lineWidth: 0.1,
      cellWidth: 'auto'
    },
    rowPageBreak: 'auto',
    headStyles: {
      fillColor: [192, 192, 192],
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      cellWidth: 'wrap',
      halign: 'center',
      valign: 'middle'
    },
    bodyStyles: {
      textColor: [0, 0, 0]
    },
    didParseCell: function (data) {
      if (data.row.section === 'head') {
        data.cell.text = data.cell.text.map(function (word: any) {
          return word.toUpperCase()
        })
      }

      if (data.cell.text[0] === 'RATA-RATA') {
        for (const key in data.row.cells) {
          data.row.cells[key].styles.fillColor = [192, 192, 192]
          data.row.cells[key].styles.fontStyle = 'bold'
        }
      }
    },
    startY: 30
  })

  autoTable(doc, {
    head: [
      [`${day}, ${date}-${month}-${year}`],
      [
        {
          content: '(.........................................)',
          styles: { minCellHeight: 8, valign: 'bottom' }
        }
      ]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    tableWidth: 50,
    margin: { left: 230 }
  })

  if (preview) {
    doc.setProperties({
      title: `${reportName}`
    })
    window.open(doc.output('bloburl'))
  } else {
    doc.save(`Laporan ${reportName}.pdf`, { returnPromise: true }).then(() => {
      console.log('pdf berhasil disimpan')
    })
  }
}

const exportToXLSX = (reportMeta: any, e: any) => {
  const workbook = new Workbook()
  const worksheet = workbook.addWorksheet(`${reportName}`)

  setHeaderStyle(worksheet, 1, 1, 'PT. PLN(Persero)')
  setHeaderStyle(
    worksheet,
    2,
    1,
    `UNIT INDUK : ${
      reportMeta.uid
        ? reportMeta.uid.name.toUpperCase()
        : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    3,
    1,
    `UNIT PELAKSANA PELAYANAN PELANGGAN : ${
      reportMeta.up3
        ? reportMeta.up3.name.toUpperCase()
        : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    4,
    1,
    `UNIT LAYANAN PELANGGAN : ${
      reportMeta.ulp
        ? reportMeta.ulp.name.toUpperCase()
        : 'Semua Unit Layanan Pelanggan'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    5,
    1,
    `REGIONAL : ${
      reportMeta.regional ? reportMeta.regional.name.toUpperCase() : 'Semua Regional'.toUpperCase()
    }`
  )

  setHeaderStyle(worksheet, 7, 1, `${reportName}`.toUpperCase(), true)
  setHeaderStyle(worksheet, 8, 1, reportMeta.periode, true)

  worksheet.mergeCells('A1:G1')
  worksheet.mergeCells('A2:G2')
  worksheet.mergeCells('A3:G3')
  worksheet.mergeCells('A4:G4')
  worksheet.mergeCells('A5:G5')
  worksheet.mergeCells('A7:G7')
  worksheet.mergeCells('A8:G8')

  exportToExcel({
    component: e.component,
    worksheet,
    autoFilterEnabled: true,
    topLeftCell: { row: 10, column: 1 },
    loadPanel: {
      enabled: false
    }
  }).then(() => {
    workbook.xlsx.writeBuffer().then((buffer: any) => {
      saveAs(new Blob([buffer], { type: 'application/octet-stream' }), `Laporan ${reportName}.xlsx`)
    })
  })

  e.cancel = true
}

const exportToDOCX = (reportMeta: any, rawData: any) => {
  exportToWord(reportMeta, formatData(rawData, reportMeta), `Laporan ${reportName}`, null)
}
export { exportToPDF, exportToXLSX, exportToDOCX }
